#ifdef __DEBUG
#include <syslog.h>
#endif
#include <stdio.h>
#include <regex.h>
#include "locals_regex.h"

/*

   Regular expressions for /var/qmail/control/locals

   Written 1999 by Robert Sander <gurubert@cs.tu-berlin.de>

   If you want to have debug output, define __DEBUG while compiling

*/

int locals_regex_comp(lr,s,len)
struct locals_regex *lr;
char *s;
int len;
{

#ifdef __DEBUG
  openlog("constmap_regex", LOG_PID, LOG_MAIL);
  syslog(LOG_INFO, "starting with s=\"%s\"", s);
#endif

  if (lr) {
    *(s+len) = '\0';
    if (regexec(&(lr->preg), s, 0, 0, 0)) {
#ifdef __DEBUG
      syslog(LOG_INFO, "NO MATCH with %s", lr->preg.buffer);
      closelog;
#endif
      *(s+len) = 'Z';
      return(locals_regex_comp(lr->next, s, len));
    } else {
#ifdef __DEBUG
      syslog(LOG_INFO, "MATCH with %s", lr->preg.buffer);
      closelog;
#endif
      *(s+len) = 'Z';
      return(!lr->not);
    }
  }
#ifdef __DEBUG
  syslog(LOG_INFO, "lr is null");
  closelog();
#endif
  return(0);
}

int locals_regex_init(lr,s,len)
     struct locals_regex *lr;
     char* s;
     int len;
{
  int i;
  struct locals_regex *act;

#ifdef __DEBUG
  openlog("locals_regex_init", LOG_PID, LOG_MAIL);
  syslog(LOG_INFO, "starting with %s, %d", s, len);
#endif

  act = lr;
  i = 0;
  while (i<len) {
    if (act) {

#ifdef __DEBUG
      syslog(LOG_INFO, "Position %d, string %s", i, (s+i));
#endif

      if (*(s+i) == '!') {
#ifdef __DEBUG
	syslog(LOG_INFO, "regex negated");
#endif
	act->not = 1;
	i++;
      } else {
	act->not = 0;
      }

      if (regcomp(&(act->preg), (s+i), REG_EXTENDED|REG_NOSUB|REG_ICASE)) {
#ifdef __DEBUG
	syslog(LOG_INFO, "regcomp failed");
	closelog();
#endif
	return(0);
      }
	
    } else {
#ifdef __DEBUG
      syslog(LOG_INFO, "act is zero");
      closelog();
#endif
      return(0);
    }

    while (s[i]) {
      i++;
    }
    i++;
    if (i<len) {
      act->next = (struct locals_regex *)malloc(sizeof(struct locals_regex));
      act = act->next;
      if (act) {
	act->next = NULL;
      }
    }
  }

#ifdef __DEBUG
  syslog(LOG_INFO, "finished");
  closelog;
#endif
  return(1);
}

void locals_regex_free(lr)
     struct locals_regex *lr;
{
  if (lr) {
#ifdef __DEBUG
    openlog("locals_regex_free", LOG_PID, LOG_MAIL);
    syslog(LOG_INFO, "starting");
    closelog();
#endif
    locals_regex_free(lr->next);
    regfree(&(lr->preg));
    free(lr);
  }
}
